from bs4 import BeautifulSoup
import requests
import random


def request_wiki(url, heading):
    response = requests.get(
        "https://escapefromtarkov.fandom.com/wiki/Category:" + url, timeout=25
    ).text
    soup = BeautifulSoup(response, "html.parser")
    h2_heading = soup.find("h2", string=f'Pages in category "{heading}"')
    return [link.text for link in h2_heading.find_next("div").find_all("a")]


def allowed_level_roll():
    message = "None"
    allowed = random.randint(0, 1) == 1
    if allowed:
        message = "Allowed"
        message += ", Trader level: " + random.choice(
            [str(random.randint(1, 4)), "Any level"]
        )
    return message


def query_tarkov_api(query):
    headers = {"Content-Type": "application/json"}
    return requests.post(
        "https://api.tarkov.dev/graphql", headers=headers, json={"query": query}
    ).json()["data"]


def tarkov_boss_info():
    """
    Returns a dict of boss spawn chances per map, and their escorts
    """

    query = """{
          maps {
            name
            bosses {
              name
              spawnChance
              escorts {
                boss {
                  name
                }
                amount {
                  count
                }
              }
              boss {
                name
                imagePortraitLink
              }
            }
          }
        }"""

    dont_care = [
        "assault",
        "Infected",
        "BEAR",
        "USEC",
        "Rogue",
    ]
    response = query_tarkov_api(query)["maps"]

    # Wasteful, but make a new dict to hold the info we care about
    levels = {}
    for level in response:
        levels[level["name"]] = {
            boss["boss"]["name"]: {
                "escorts": sum(
                    escort["amount"][0]["count"] for escort in boss["escorts"]
                ),
                "spawnChance": f"**{str(round(boss['spawnChance'] * 100, 2))}%**",
                "picture": boss["boss"]["imagePortraitLink"],
            }
            for boss in level["bosses"]
            if boss["boss"]["name"] not in dont_care
        }

    return levels


def compare_boss_spawns(known_spawns, spawns_from_api):
    changes = {}  # To store the changes

    for level in known_spawns:
        level_changes = {}  # Store changes for each level
        for boss in known_spawns.get(level, {}):
            boss_info_1 = known_spawns[level].get(boss, {})
            boss_info_2 = spawns_from_api[level].get(boss, {})

            known_spawn_rate = boss_info_1.get("spawnChance", "N/A")
            spawn_rate_from_api = boss_info_2.get("spawnChance", "N/A")

            if known_spawn_rate != spawn_rate_from_api:
                level_changes[boss] = f"{known_spawn_rate} -> {spawn_rate_from_api}"

        if level_changes:
            changes[level] = level_changes

    return changes
